@@ -1,6 +1,4 @@
 mod builder;
-mod record;
-mod sam_record;
 
 pub use self::builder::Builder;
 
@@ -10,11 +8,15 @@ use noodles_bgzf as bgzf;
 use noodles_sam as sam;
 use tokio::io::{self, AsyncWrite, AsyncWriteExt};
 
-use crate::Record;
+use crate::{
+    writer::{alignment_record::encode_alignment_record, record::encode_record},
+    Record,
+};
 
 /// An async BAM writer.
 pub struct Writer<W> {
     inner: W,
+    buf: Vec<u8>,
 }
 
 impl<W> Writer<W>
@@ -148,7 +150,17 @@ where
     /// # }
     /// ```
     pub async fn write_record(&mut self, record: &Record) -> io::Result<()> {
-        record::write_record(&mut self.inner, record).await
+        encode_record(&mut self.buf, record)?;
+
+        let block_size = u32::try_from(self.buf.len())
+            .map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?;
+        self.inner.write_u32_le(block_size).await?;
+
+        self.inner.write_all(&self.buf).await?;
+
+        self.buf.clear();
+
+        Ok(())
     }
 
     /// Writes a SAM record.
@@ -176,7 +188,17 @@ where
         reference_sequences: &sam::header::ReferenceSequences,
         record: &sam::Record,
     ) -> io::Result<()> {
-        sam_record::write_sam_record(&mut self.inner, reference_sequences, record).await
+        encode_alignment_record(&mut self.buf, reference_sequences, record)?;
+
+        let block_size = u32::try_from(self.buf.len())
+            .map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?;
+        self.inner.write_u32_le(block_size).await?;
+
+        self.inner.write_all(&self.buf).await?;
+
+        self.buf.clear();
+
+        Ok(())
     }
 }
 
@@ -212,7 +234,10 @@ where
 
 impl<W> From<W> for Writer<W> {
     fn from(inner: W) -> Self {
-        Self { inner }
+        Self {
+            inner,
+            buf: Vec::new(),
+        }
     }
 }
 
