@@ -10,10 +10,19 @@ use std::{
     ops::{Bound, RangeBounds},
 };
 
+use noodles_core::Position;
 use noodles_csi::{
     binning_index::optimize_chunks, index::reference_sequence::bin::Chunk, BinningIndex,
 };
 
+const MIN_SHIFT: u8 = 14;
+const DEPTH: u8 = 5;
+
+const MAX_POSITION: Position = match Position::new((1 << (MIN_SHIFT + 3 * DEPTH)) - 1) {
+    Some(position) => position,
+    None => panic!(),
+};
+
 /// A BAM index.
 #[derive(Clone, Debug, Default, Eq, PartialEq)]
 pub struct Index {
@@ -110,7 +119,7 @@ impl BinningIndex<ReferenceSequence> for Index {
 
     fn query<B>(&self, reference_sequence_id: usize, interval: B) -> io::Result<Vec<Chunk>>
     where
-        B: RangeBounds<i32> + Clone,
+        B: RangeBounds<Position> + Clone,
     {
         let reference_sequence = self
             .reference_sequences()
@@ -132,15 +141,46 @@ impl BinningIndex<ReferenceSequence> for Index {
             .copied()
             .collect();
 
-        let start = match interval.start_bound() {
-            Bound::Included(s) => *s,
-            Bound::Excluded(s) => *s + 1,
-            Bound::Unbounded => 1,
-        };
-
+        let (start, _) = resolve_interval(interval)?;
         let min_offset = reference_sequence.min_offset(start);
         let merged_chunks = optimize_chunks(&chunks, min_offset);
 
         Ok(merged_chunks)
     }
 }
+
+fn resolve_interval<B>(interval: B) -> io::Result<(Position, Position)>
+where
+    B: RangeBounds<Position>,
+{
+    let start = match interval.start_bound() {
+        Bound::Included(position) => *position,
+        Bound::Excluded(position) => position
+            .checked_add(1)
+            .expect("attempt to add with overflow"),
+        Bound::Unbounded => Position::MIN,
+    };
+
+    if start > MAX_POSITION {
+        return Err(io::Error::new(
+            io::ErrorKind::InvalidInput,
+            "invalid start bound",
+        ));
+    }
+
+    let end = match interval.end_bound() {
+        Bound::Included(position) => *position,
+        Bound::Excluded(position) => Position::try_from(usize::from(*position) - 1)
+            .map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?,
+        Bound::Unbounded => MAX_POSITION,
+    };
+
+    if end > MAX_POSITION {
+        Err(io::Error::new(
+            io::ErrorKind::InvalidInput,
+            "invalid end bound",
+        ))
+    } else {
+        Ok((start, end))
+    }
+}
