@@ -1,11 +1,13 @@
 use std::{cmp, collections::HashMap, io};
 
 use noodles_bgzf as bgzf;
+use noodles_core::Position;
 use noodles_csi::index::reference_sequence::bin::Chunk;
+use noodles_sam::record::Flags;
 
-use crate::Record;
+use crate::writer::record::region_to_bin;
 
-use super::{bin, Bin, Metadata, ReferenceSequence, WINDOW_SIZE};
+use super::{bin, Bin, Metadata, ReferenceSequence, MIN_SHIFT};
 
 // § 5.2 The BAI index format for BAM files (2020-07-19)
 const MAX_INTERVAL_COUNT: usize = 131072;
@@ -21,10 +23,16 @@ pub struct Builder {
 }
 
 impl Builder {
-    pub fn add_record(&mut self, record: &Record, chunk: Chunk) -> io::Result<()> {
-        self.update_bins(record, chunk);
-        self.update_linear_index(record, chunk)?;
-        self.update_metadata(record, chunk);
+    pub fn add_record(
+        &mut self,
+        start: Position,
+        end: Position,
+        flags: Flags,
+        chunk: Chunk,
+    ) -> io::Result<()> {
+        self.update_bins(start, end, chunk)?;
+        self.update_linear_index(start, end, chunk);
+        self.update_metadata(flags, chunk);
         Ok(())
     }
 
@@ -55,8 +63,8 @@ impl Builder {
         ReferenceSequence::new(bins, intervals, Some(metadata))
     }
 
-    fn update_bins(&mut self, record: &Record, chunk: Chunk) {
-        let bin_id = u32::from(record.bin());
+    fn update_bins(&mut self, start: Position, end: Position, chunk: Chunk) -> io::Result<()> {
+        let bin_id = region_to_bin(start, end).map(u32::from)?;
 
         let builder = self.bin_builders.entry(bin_id).or_insert_with(|| {
             let mut builder = Bin::builder();
@@ -65,15 +73,15 @@ impl Builder {
         });
 
         builder.add_chunk(chunk);
+
+        Ok(())
     }
 
-    fn update_linear_index(&mut self, record: &Record, chunk: Chunk) -> io::Result<()> {
-        let start = record.position().map(i32::from).expect("missing position");
-        let reference_len = record.cigar().reference_len().map(|len| len as i32)?;
-        let end = start + reference_len - 1;
+    fn update_linear_index(&mut self, start: Position, end: Position, chunk: Chunk) {
+        const WINDOW_SIZE: usize = 1 << MIN_SHIFT;
 
-        let linear_index_start_offset = ((start - 1) / WINDOW_SIZE) as usize;
-        let linear_index_end_offset = ((end - 1) / WINDOW_SIZE) as usize;
+        let linear_index_start_offset = (usize::from(start) - 1) / WINDOW_SIZE;
+        let linear_index_end_offset = (usize::from(end) - 1) / WINDOW_SIZE;
 
         if linear_index_end_offset >= self.intervals.len() {
             self.intervals
@@ -83,12 +91,10 @@ impl Builder {
         for i in linear_index_start_offset..=linear_index_end_offset {
             self.intervals[i].get_or_insert(chunk.start());
         }
-
-        Ok(())
     }
 
-    fn update_metadata(&mut self, record: &Record, chunk: Chunk) {
-        if record.flags().is_unmapped() {
+    fn update_metadata(&mut self, flags: Flags, chunk: Chunk) {
+        if flags.is_unmapped() {
             self.unmapped_record_count += 1;
         } else {
             self.mapped_record_count += 1;
@@ -114,54 +120,28 @@ impl Default for Builder {
 
 #[cfg(test)]
 mod tests {
-    use noodles_sam::{
-        self as sam,
-        record::{Flags, Position},
-    };
+    use noodles_core::Position;
 
     use super::*;
 
     #[test]
     fn test_build() -> Result<(), Box<dyn std::error::Error>> {
-        use sam::header::reference_sequence;
-
-        let reference_sequences = [("sq0".parse()?, 8)]
-            .into_iter()
-            .map(|(name, len): (reference_sequence::Name, i32)| {
-                let sn = name.to_string();
-                sam::header::ReferenceSequence::new(name, len).map(|rs| (sn, rs))
-            })
-            .collect::<Result<_, _>>()?;
-
         let mut builder = Builder::default();
 
-        let record = Record::try_from_sam_record(
-            &reference_sequences,
-            &sam::Record::builder()
-                .set_flags(Flags::empty())
-                .set_position(Position::try_from(2)?)
-                .set_cigar("4M".parse()?)
-                .build()?,
-        )?;
-
         builder.add_record(
-            &record,
+            Position::try_from(2)?,
+            Position::try_from(5)?,
+            Flags::empty(),
             Chunk::new(
                 bgzf::VirtualPosition::from(55),
                 bgzf::VirtualPosition::from(89),
             ),
         )?;
 
-        let record = Record::try_from_sam_record(
-            &reference_sequences,
-            &sam::Record::builder()
-                .set_position(Position::try_from(6)?)
-                .set_cigar("2M".parse()?)
-                .build()?,
-        )?;
-
         builder.add_record(
-            &record,
+            Position::try_from(6)?,
+            Position::try_from(7)?,
+            Flags::UNMAPPED,
             Chunk::new(
                 bgzf::VirtualPosition::from(89),
                 bgzf::VirtualPosition::from(144),
