@@ -16,7 +16,11 @@ use byteorder::{LittleEndian, ReadBytesExt};
 use noodles_bgzf as bgzf;
 use noodles_core::{region::Interval, Region};
 use noodles_csi::{binning_index::ReferenceSequenceExt, BinningIndex};
-use noodles_sam::header::{ReferenceSequence, ReferenceSequences};
+use noodles_fasta as fasta;
+use noodles_sam::{
+    self as sam,
+    header::{ReferenceSequence, ReferenceSequences},
+};
 
 use self::record::read_record;
 use super::{bai, Record, MAGIC_NUMBER};
@@ -296,7 +300,7 @@ where
     ///
     /// let reference_sequences = header.reference_sequences();
     /// let index = bai::read("sample.bam.bai")?;
-    /// let region = Region::mapped("sq0", 17711..=28657);
+    /// let region = "sq0:8-13".parse()?;
     /// let query = reader.query(reference_sequences, &index, &region)?;
     ///
     /// for result in query {
@@ -310,14 +314,21 @@ where
         reference_sequences: &ReferenceSequences,
         index: &I,
         region: &Region,
-    ) -> io::Result<Query<'_, R>>
+    ) -> io::Result<Query<'_, R, Interval>>
     where
         I: BinningIndex<RS>,
         RS: ReferenceSequenceExt,
     {
-        let (reference_sequence_id, interval) = resolve_region(reference_sequences, region)?;
-        let chunks = index.query(reference_sequence_id, interval)?;
-        Ok(Query::new(self, chunks, reference_sequence_id, interval))
+        let reference_sequence_id = resolve_region(reference_sequences, region)?;
+
+        let chunks = index.query(reference_sequence_id, region.interval())?;
+
+        Ok(Query::new(
+            self,
+            chunks,
+            reference_sequence_id,
+            region.interval(),
+        ))
     }
 
     /// Returns an iterator of unmapped records after querying for the unmapped region.
@@ -358,6 +369,36 @@ impl<R> From<R> for Reader<R> {
     }
 }
 
+impl<R> sam::AlignmentReader for Reader<R>
+where
+    R: Read,
+{
+    fn read_alignment_header(&mut self) -> io::Result<sam::Header> {
+        read_magic(&mut self.inner)?;
+
+        let header = read_header(&mut self.inner).and_then(|s| {
+            s.parse()
+                .map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))
+        })?;
+
+        self.read_reference_sequences()?;
+
+        Ok(header)
+    }
+
+    fn alignment_records<'a>(
+        &'a mut self,
+        _: &'a fasta::Repository,
+        _: &'a sam::Header,
+    ) -> Box<dyn Iterator<Item = io::Result<Box<dyn sam::AlignmentRecord>>> + 'a> {
+        Box::new(
+            self.records().map(|result| {
+                result.map(|record| Box::new(record) as Box<dyn sam::AlignmentRecord>)
+            }),
+        )
+    }
+}
+
 fn read_magic<R>(reader: &mut R) -> io::Result<()>
 where
     R: Read,
@@ -449,9 +490,10 @@ pub(crate) fn bytes_with_nul_to_string(buf: &[u8]) -> io::Result<String> {
 pub(crate) fn resolve_region(
     reference_sequences: &ReferenceSequences,
     region: &Region,
-) -> io::Result<(usize, Interval)> {
-    if let Some(r) = region.as_mapped() {
-        let i = reference_sequences.get_index_of(r.name()).ok_or_else(|| {
+) -> io::Result<usize> {
+    reference_sequences
+        .get_index_of(region.name())
+        .ok_or_else(|| {
             io::Error::new(
                 io::ErrorKind::InvalidInput,
                 format!(
@@ -459,15 +501,7 @@ pub(crate) fn resolve_region(
                     region
                 ),
             )
-        })?;
-
-        Ok((i, r.interval()))
-    } else {
-        Err(io::Error::new(
-            io::ErrorKind::InvalidData,
-            "region is not mapped",
-        ))
-    }
+        })
 }
 
 #[cfg(test)]
