@@ -1,8 +1,8 @@
-use std::{io, str};
+use std::io;
 
-use noodles_sam::{self as sam, RecordExt};
+use noodles_sam::{self as sam, AlignmentRecord};
 
-use super::{Record, ReferenceSequenceId};
+use super::Record;
 
 impl Record {
     /// Converts a SAM record to a BAM record.
@@ -25,14 +25,16 @@ impl Record {
         reference_sequences: &sam::header::ReferenceSequences,
         sam_record: &sam::Record,
     ) -> io::Result<Self> {
-        use crate::{reader::record::read_record, writer::sam_record::write_sam_record};
+        use crate::{
+            reader::record::decode_record, writer::alignment_record::encode_alignment_record,
+        };
 
         let mut buf = Vec::new();
-        write_sam_record(&mut buf, reference_sequences, sam_record)?;
+        encode_alignment_record(&mut buf, reference_sequences, sam_record)?;
 
         let mut reader = &buf[..];
         let mut record = Self::default();
-        read_record(&mut reader, &mut Vec::new(), &mut record)?;
+        decode_record(&mut reader, &mut record)?;
 
         Ok(record)
     }
@@ -60,15 +62,8 @@ impl Record {
     ) -> io::Result<sam::Record> {
         let mut builder = sam::Record::builder();
 
-        let raw_read_name = str::from_utf8(self.read_name())
-            .map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?;
-
-        if raw_read_name != "*" {
-            let read_name = raw_read_name
-                .parse()
-                .map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?;
-
-            builder = builder.set_read_name(read_name);
+        if let Some(read_name) = self.read_name() {
+            builder = builder.set_read_name(read_name.clone());
         }
 
         builder = builder.set_flags(self.flags());
@@ -87,8 +82,9 @@ impl Record {
             builder = builder.set_mapping_quality(mapping_quality);
         }
 
-        let cigar = self.cigar().try_into()?;
-        builder = builder.set_cigar(cigar);
+        if !self.cigar().is_empty() {
+            builder = builder.set_cigar(self.cigar().clone());
+        }
 
         if let Some(mate_reference_sequence_name) =
             get_reference_sequence_name(reference_sequences, self.mate_reference_sequence_id())?
@@ -103,37 +99,26 @@ impl Record {
         builder = builder.set_template_length(self.template_length());
 
         if !self.sequence().is_empty() {
-            builder = builder.set_sequence(self.sequence().into());
+            builder = builder.set_sequence(self.sequence().clone());
         }
 
         if !self.quality_scores().is_empty() {
-            let quality_scores = self
-                .quality_scores()
-                .try_into()
-                .map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))?;
-
-            builder = builder.set_quality_scores(quality_scores);
+            builder = builder.set_quality_scores(self.quality_scores().clone());
         }
 
-        let data = self
-            .data()
-            .try_into()
-            .map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))?;
+        if !self.data().is_empty() {
+            builder = builder.set_data(self.data().clone());
+        }
 
-        builder = builder.set_data(data);
-
-        builder
-            .build()
-            .map_err(|e| io::Error::new(io::ErrorKind::InvalidData, e))
+        Ok(builder.build())
     }
 }
 
 fn get_reference_sequence_name(
     reference_sequences: &sam::header::ReferenceSequences,
-    reference_sequence_id: Option<ReferenceSequenceId>,
+    reference_sequence_id: Option<usize>,
 ) -> io::Result<Option<sam::record::ReferenceSequenceName>> {
     reference_sequence_id
-        .map(usize::from)
         .map(|id| {
             reference_sequences
                 .get_index(id)
@@ -147,6 +132,8 @@ fn get_reference_sequence_name(
 
 #[cfg(test)]
 mod tests {
+    use noodles_core::Position;
+
     use super::*;
 
     fn build_reference_sequences(
@@ -168,13 +155,9 @@ mod tests {
     }
 
     fn build_record() -> Result<Record, Box<dyn std::error::Error>> {
-        use sam::record::{
-            cigar::op::Kind, quality_scores::Score, Flags, MappingQuality, Position,
-        };
+        use sam::record::{Flags, MappingQuality};
 
-        use crate::record::{cigar::Op, sequence::Base, Cigar, Data, QualityScores, Sequence};
-
-        let reference_sequence_id = ReferenceSequenceId::from(1);
+        let reference_sequence_id = 1;
 
         let record = Record::builder()
             .set_reference_sequence_id(reference_sequence_id)
@@ -184,34 +167,18 @@ mod tests {
             .set_mate_reference_sequence_id(reference_sequence_id)
             .set_mate_position(Position::try_from(61153)?)
             .set_template_length(166)
-            .set_read_name(b"r0".to_vec())
-            .set_cigar(Cigar::from(vec![Op::new(Kind::Match, 4)?]))
-            .set_sequence(Sequence::from(vec![Base::A, Base::T, Base::G, Base::C]))
-            .set_quality_scores(QualityScores::from(vec![
-                Score::try_from('@')?,
-                Score::try_from('>')?,
-                Score::try_from('?')?,
-                Score::try_from('A')?,
-            ]))
-            .set_data(Data::try_from(vec![
-                0x4e, 0x4d, 0x43, 0x00, // NM:i:0
-                0x50, 0x47, 0x5a, 0x53, 0x4e, 0x41, 0x50, 0x00, // PG:Z:SNAP
-            ])?)
-            .build()?;
+            .set_read_name("r0".parse()?)
+            .set_cigar("4M".parse()?)
+            .set_sequence("ATGC".parse()?)
+            .set_quality_scores("@>?A".parse()?)
+            .set_data("NM:i:0\tPG:Z:SNAP".parse()?)
+            .build();
 
         Ok(record)
     }
 
     #[test]
     fn test_try_into_sam_record() -> Result<(), Box<dyn std::error::Error>> {
-        use sam::record::{
-            cigar::{op, Op},
-            data::{
-                field::{Tag, Value},
-                Field,
-            },
-        };
-
         let bam_record = build_record()?;
         let reference_sequences = build_reference_sequences()?;
         let actual = bam_record.try_into_sam_record(&reference_sequences)?;
@@ -220,19 +187,16 @@ mod tests {
             .set_read_name("r0".parse()?)
             .set_flags(sam::record::Flags::SEGMENTED | sam::record::Flags::FIRST_SEGMENT)
             .set_reference_sequence_name("sq1".parse()?)
-            .set_position(sam::record::Position::try_from(61062)?)
+            .set_position(Position::try_from(61062)?)
             .set_mapping_quality(sam::record::MappingQuality::try_from(12)?)
-            .set_cigar(sam::record::Cigar::from(vec![Op::new(op::Kind::Match, 4)]))
+            .set_cigar("4M".parse()?)
             .set_mate_reference_sequence_name("sq1".parse()?)
-            .set_mate_position(sam::record::Position::try_from(61153)?)
+            .set_mate_position(Position::try_from(61153)?)
             .set_template_length(166)
             .set_sequence("ATGC".parse()?)
             .set_quality_scores("@>?A".parse()?)
-            .set_data(sam::record::Data::try_from(vec![
-                Field::new(Tag::EditDistance, Value::Int(0)),
-                Field::new(Tag::Program, Value::String(String::from("SNAP"))),
-            ])?)
-            .build()?;
+            .set_data("NM:i:0\tPG:Z:SNAP".parse()?)
+            .build();
 
         assert_eq!(actual, expected);
 
