@@ -1,5 +1,5 @@
-mod record;
-pub(crate) mod sam_record;
+pub(crate) mod alignment_record;
+pub(crate) mod record;
 
 use std::{
     ffi::CString,
@@ -13,6 +13,7 @@ use noodles_sam::{
     header::{ReferenceSequence, ReferenceSequences},
 };
 
+use self::{alignment_record::encode_alignment_record, record::encode_record};
 use super::Record;
 
 /// A BAM writer.
@@ -29,16 +30,17 @@ use super::Record;
 ///
 /// let mut writer = bam::Writer::new(Vec::new());
 ///
-/// let header = sam::Header::builder().add_comment("noodles-bam").build();
+/// let header = sam::Header::default();
 /// writer.write_header(&header)?;
 /// writer.write_reference_sequences(header.reference_sequences())?;
 ///
-/// let record = sam::Record::default();
-/// writer.write_sam_record(header.reference_sequences(), &record)?;
+/// let record = bam::Record::default();
+/// writer.write_record(&record)?;
 /// # Ok::<(), io::Error>(())
 /// ```
 pub struct Writer<W> {
     inner: W,
+    buf: Vec<u8>,
 }
 
 impl<W> Writer<W>
@@ -145,11 +147,22 @@ where
     /// # Ok::<(), io::Error>(())
     /// ```
     pub fn write_record(&mut self, record: &Record) -> io::Result<()> {
-        record::write_record(&mut self.inner, record)
+        encode_record(&mut self.buf, record)?;
+
+        let block_size = u32::try_from(self.buf.len())
+            .map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?;
+        self.inner.write_u32::<LittleEndian>(block_size)?;
+
+        self.inner.write_all(&self.buf)?;
+
+        self.buf.clear();
+
+        Ok(())
     }
 
     /// Writes a SAM record.
     ///
+    ///
     /// # Examples
     ///
     /// ```
@@ -159,17 +172,30 @@ where
     ///
     /// let mut writer = bam::Writer::new(Vec::new());
     ///
-    /// let reference_sequences = sam::header::ReferenceSequences::new();
+    /// let header = sam::Header::default();
     /// let record = sam::Record::default();
-    /// writer.write_sam_record(&reference_sequences, &record)?;
+    /// writer.write_sam_record(header.reference_sequences(), &record)?;
     /// # Ok::<(), io::Error>(())
-    /// ```
+    #[deprecated(
+        since = "0.17.0",
+        note = "Use `Writer::write_alignment_record` instead."
+    )]
     pub fn write_sam_record(
         &mut self,
         reference_sequences: &ReferenceSequences,
         record: &sam::Record,
     ) -> io::Result<()> {
-        sam_record::write_sam_record(&mut self.inner, reference_sequences, record)
+        encode_alignment_record(&mut self.buf, reference_sequences, record)?;
+
+        let block_size = u32::try_from(self.buf.len())
+            .map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?;
+        self.inner.write_u32::<LittleEndian>(block_size)?;
+
+        self.inner.write_all(&self.buf)?;
+
+        self.buf.clear();
+
+        Ok(())
     }
 }
 
@@ -212,7 +238,43 @@ where
 
 impl<W> From<W> for Writer<W> {
     fn from(inner: W) -> Self {
-        Self { inner }
+        Self {
+            inner,
+            buf: Vec::new(),
+        }
+    }
+}
+
+impl<W> sam::AlignmentWriter for Writer<W>
+where
+    W: Write,
+{
+    fn write_alignment_header(&mut self, header: &sam::Header) -> io::Result<()> {
+        self.write_header(header)?;
+        self.write_reference_sequences(header.reference_sequences())?;
+        Ok(())
+    }
+
+    fn write_alignment_record(
+        &mut self,
+        header: &sam::Header,
+        record: &dyn sam::AlignmentRecord,
+    ) -> io::Result<()> {
+        encode_alignment_record(&mut self.buf, header.reference_sequences(), record)?;
+
+        let block_size = u32::try_from(self.buf.len())
+            .map_err(|e| io::Error::new(io::ErrorKind::InvalidInput, e))?;
+        self.inner.write_u32::<LittleEndian>(block_size)?;
+
+        self.inner.write_all(&self.buf)?;
+
+        self.buf.clear();
+
+        Ok(())
+    }
+
+    fn finish(&mut self, _: &sam::Header) -> io::Result<()> {
+        Ok(())
     }
 }
 
@@ -276,9 +338,9 @@ where
 
 #[cfg(test)]
 mod tests {
-    use noodles_sam::record::Data;
+    use noodles_sam::{record::Data, AlignmentRecord, AlignmentWriter};
 
-    use crate::{record::sequence::Base, Reader, Record};
+    use crate::{Reader, Record};
 
     use super::*;
 
@@ -332,12 +394,12 @@ mod tests {
     }
 
     #[test]
-    fn test_write_sam_record() -> Result<(), Box<dyn std::error::Error>> {
+    fn test_write_alignment_record() -> Result<(), Box<dyn std::error::Error>> {
         let mut writer = Writer::new(Vec::new());
 
         let header = sam::Header::default();
         let sam_record = sam::Record::default();
-        writer.write_sam_record(header.reference_sequences(), &sam_record)?;
+        writer.write_alignment_record(&header, &sam_record)?;
         writer.try_finish()?;
 
         let mut reader = Reader::new(writer.get_ref().get_ref().as_slice());
@@ -345,7 +407,7 @@ mod tests {
         let mut record = Record::default();
         reader.read_record(&mut record)?;
 
-        assert_eq!(record.read_name(), b"*");
+        assert!(record.read_name().is_none());
         assert_eq!(record.flags(), sam::record::Flags::UNMAPPED);
         assert!(record.reference_sequence_id().is_none());
         assert!(record.position().is_none());
@@ -362,43 +424,37 @@ mod tests {
     }
 
     #[test]
-    fn test_write_sam_record_with_sequence_length_less_than_quality_scores_length(
+    fn test_write_alignment_record_with_sequence_length_less_than_quality_scores_length(
     ) -> Result<(), Box<dyn std::error::Error>> {
         let mut writer = Writer::new(Vec::new());
 
         let header = sam::Header::default();
 
-        let mut record = sam::Record::builder().set_sequence("AT".parse()?).build()?;
+        let mut record = sam::Record::builder().set_sequence("AT".parse()?).build();
         *record.quality_scores_mut() = "NDLS".parse()?;
 
-        assert!(writer
-            .write_sam_record(header.reference_sequences(), &record)
-            .is_err());
+        assert!(writer.write_alignment_record(&header, &record).is_err());
 
         Ok(())
     }
 
     #[test]
-    fn test_write_sam_record_with_sequence_length_greater_than_quality_scores_length(
+    fn test_write_alignment_record_with_sequence_length_greater_than_quality_scores_length(
     ) -> Result<(), Box<dyn std::error::Error>> {
         let mut writer = Writer::new(Vec::new());
 
         let header = sam::Header::default();
 
-        let mut record = sam::Record::builder()
-            .set_sequence("ATCG".parse()?)
-            .build()?;
+        let mut record = sam::Record::builder().set_sequence("ATCG".parse()?).build();
         *record.quality_scores_mut() = "ND".parse()?;
 
-        assert!(writer
-            .write_sam_record(header.reference_sequences(), &record)
-            .is_err());
+        assert!(writer.write_alignment_record(&header, &record).is_err());
 
         Ok(())
     }
 
     #[test]
-    fn test_write_sam_record_with_no_sequence_and_with_quality_scores(
+    fn test_write_alignment_record_with_no_sequence_and_with_quality_scores(
     ) -> Result<(), Box<dyn std::error::Error>> {
         let mut writer = Writer::new(Vec::new());
 
@@ -406,24 +462,20 @@ mod tests {
         let mut record = sam::Record::default();
         *record.quality_scores_mut() = "NDLS".parse()?;
 
-        assert!(writer
-            .write_sam_record(header.reference_sequences(), &record)
-            .is_err());
+        assert!(writer.write_alignment_record(&header, &record).is_err());
 
         Ok(())
     }
 
     #[test]
-    fn test_write_sam_record_with_sequence_and_no_quality_scores(
+    fn test_write_alignment_record_with_sequence_and_no_quality_scores(
     ) -> Result<(), Box<dyn std::error::Error>> {
         let mut writer = Writer::new(Vec::new());
 
         let header = sam::Header::default();
-        let sam_record = sam::Record::builder()
-            .set_sequence("ATCG".parse()?)
-            .build()?;
+        let sam_record = sam::Record::builder().set_sequence("ATCG".parse()?).build();
 
-        writer.write_sam_record(header.reference_sequences(), &sam_record)?;
+        writer.write_alignment_record(&header, &sam_record)?;
         writer.try_finish()?;
 
         let mut reader = Reader::new(writer.get_ref().get_ref().as_slice());
@@ -431,17 +483,16 @@ mod tests {
         let mut record = Record::default();
         reader.read_record(&mut record)?;
 
-        let actual: Vec<_> = record.sequence().bases().collect();
-        let expected = [Base::A, Base::T, Base::C, Base::G];
-        assert_eq!(actual, expected);
+        let expected = "ATCG".parse()?;
+        assert_eq!(record.sequence(), &expected);
 
-        assert_eq!(record.quality_scores().as_ref(), [255, 255, 255, 255]);
+        assert!(record.quality_scores().is_empty());
 
         Ok(())
     }
 
     #[test]
-    fn test_write_sam_record_with_sequence_and_quality_scores(
+    fn test_write_alignment_record_with_sequence_and_quality_scores(
     ) -> Result<(), Box<dyn std::error::Error>> {
         let mut writer = Writer::new(Vec::new());
 
@@ -449,9 +500,9 @@ mod tests {
         let sam_record = sam::Record::builder()
             .set_sequence("ATCG".parse()?)
             .set_quality_scores("NDLS".parse()?)
-            .build()?;
+            .build();
 
-        writer.write_sam_record(header.reference_sequences(), &sam_record)?;
+        writer.write_alignment_record(&header, &sam_record)?;
         writer.try_finish()?;
 
         let mut reader = Reader::new(writer.get_ref().get_ref().as_slice());
@@ -459,35 +510,32 @@ mod tests {
         let mut record = Record::default();
         reader.read_record(&mut record)?;
 
-        let actual: Vec<_> = record.sequence().bases().collect();
-        let expected = [Base::A, Base::T, Base::C, Base::G];
-        assert_eq!(actual, expected);
+        let expected = "ATCG".parse()?;
+        assert_eq!(record.sequence(), &expected);
 
-        assert_eq!(record.quality_scores().as_ref(), [45, 35, 43, 50]);
+        assert_eq!(record.quality_scores(), sam_record.quality_scores());
 
         Ok(())
     }
 
     #[test]
-    fn test_write_sam_record_with_data() -> Result<(), Box<dyn std::error::Error>> {
+    fn test_write_alignment_record_with_data() -> Result<(), Box<dyn std::error::Error>> {
         use noodles_sam::record::data::{
-            field::{Tag as SamTag, Value as SamValue},
-            Field as SamField,
+            field::{Tag, Value},
+            Field,
         };
 
-        use crate::record::data::{field::Value, Field};
-
         let mut writer = Writer::new(Vec::new());
 
         let header = sam::Header::default();
         let sam_record = sam::Record::builder()
             .set_data(Data::try_from(vec![
-                SamField::new(SamTag::ReadGroup, SamValue::String(String::from("rg0"))),
-                SamField::new(SamTag::AlignmentHitCount, SamValue::Int(1)),
+                Field::new(Tag::ReadGroup, Value::String(String::from("rg0"))),
+                Field::new(Tag::AlignmentHitCount, Value::UInt8(1)),
             ])?)
-            .build()?;
+            .build();
 
-        writer.write_sam_record(header.reference_sequences(), &sam_record)?;
+        writer.write_alignment_record(&header, &sam_record)?;
         writer.try_finish()?;
 
         let mut reader = Reader::new(writer.get_ref().get_ref().as_slice());
@@ -499,16 +547,16 @@ mod tests {
         let mut fields = bam_data.values();
 
         assert_eq!(
-            fields.next().transpose()?,
-            Some(Field::new(
-                SamTag::ReadGroup,
+            fields.next(),
+            Some(&Field::new(
+                Tag::ReadGroup,
                 Value::String(String::from("rg0"))
-            ),)
+            ))
         );
 
         assert_eq!(
-            fields.next().transpose()?,
-            Some(Field::new(SamTag::AlignmentHitCount, Value::UInt8(1)))
+            fields.next(),
+            Some(&Field::new(Tag::AlignmentHitCount, Value::UInt8(1)))
         );
 
         assert!(fields.next().is_none());
